package com.tptj.demo.hg.background.quick.ui;

import com.fr.base.background.AbstractBackground;
import com.fr.general.Background;
import com.fr.json.JSONException;
import com.fr.json.JSONObject;
import com.fr.stable.StableUtils;
import com.fr.stable.StringUtils;
import com.fr.stable.bridge.ObjectHolder;
import com.fr.stable.web.Repository;
import com.fr.stable.xml.XMLPrintWriter;
import com.fr.stable.xml.XMLableReader;
import java.awt.*;


/**
 * @author 秃破天际
 * @version 10.0
 * Created by 秃破天际 on 2021-02-25
 * 注：因为接口设计的问题，不要直接集成产品内置的背景对象，重新编辑的时候会被转换成他的继承的对象
 * 具体可以参见：BackgroundPane#supportKindsOfBackgroundUI、populateBean
 **/
public class DemoBackground extends AbstractBackground {

    private String theme;
    private String tag;

    public String getTheme() {
        return theme;
    }

    public void setTheme(String theme) {
        this.theme = theme;
    }

    public String getTag() {
        return tag;
    }

    public void setTag(String tag) {
        this.tag = tag;
    }

    private Color getColor() {
        return ThemeManager.getInstance().getColor(theme,tag);
    }

    @Override
    /**
     * 根据指定的画图对象和几何图形来画颜色背景
     *
     * @param g     画图对象
     * @param shape 几何图形
     */
    public void paint(Graphics g, Shape shape) {
        Paint paint = this.getColor();
        if (paint == null) {
            return;
        }
        // shape类型 Rectangle2D
        Graphics2D g2d = (Graphics2D) g;
        Paint oldPaint = g2d.getPaint();
        g2d.setPaint(paint);
        g2d.fill(shape);
        g2d.setPaint(oldPaint);
    }

    @Override
    public void drawWithGradientLine(Graphics g, Shape shape) {
        Paint paint = this.getColor();
        if (paint == null) {
            return;
        }
        // shape类型 Rectangle2D
        Graphics2D g2d = (Graphics2D) g;
        Paint oldPaint = g2d.getPaint();
        g2d.setPaint(paint);
        g2d.draw(shape);
        g2d.setPaint(oldPaint);
    }

    @Override
    public String getBackgroundType() {
        return "Theme";
    }

    @Override
    public void paint(Graphics g, Repository repo, Shape shape) {
        paint(g, shape);
    }

    /**
     * 布局变化
     *
     * @param width  背景宽度
     * @param height 背景高度
     */
    @Override
    public void layoutDidChange(int width, int height) {
        super.layoutDidChange(width, height);
        System.out.println();
    }

    /**
     * 修正哈希码
     *
     * @param code 已经计算好的哈希码
     * @return 哈希码
     */
    @Override
    public int fixHashCode(int code) {
        return code ^ (theme+tag).hashCode() ;
    }

    /**
     * 克隆
     *
     * @return 克隆生成的对象
     * @throws CloneNotSupportedException
     */
    @Override
    public Object clone() throws CloneNotSupportedException {
        DemoBackground obj =  (DemoBackground)super.clone();
        obj.tag = tag;
        obj.theme = theme;
        return obj;
    }


    /**
     * 将背景输出成JSON对象
     *
     * @return 表示背景的JSON对象
     * @throws JSONException
     */
    @Override
    public JSONObject toJSONObject() throws JSONException {
        JSONObject jo = super.toJSONObject();
        jo.put("color", StableUtils.javaColorToCSSColor(getColor()));
        return jo;
    }

    @Override
    public boolean equals(Object object) {
        return object instanceof DemoBackground &&
                StringUtils.equals(((DemoBackground) object).theme, theme) &&
                StringUtils.equals(((DemoBackground) object).tag, tag);
    }

    @Override
    public JSONObject toJSONObject(Repository repo) throws JSONException{
        JSONObject jo = super.toJSONObject(repo);
        jo.put("color", StableUtils.javaColorToCSSColor(getColor()));
        return jo;
    }
    @Override
    public JSONObject toJSONObject(Repository repo, Dimension size) throws JSONException {
        JSONObject jo = super.toJSONObject(repo, size);
        jo.put("color", StableUtils.javaColorToCSSColor(getColor()));
        return jo;
    }

    @Override
    public Background readAdditionalAttr(XMLableReader reader) {
        DemoBackground item = new DemoBackground();
        item.theme = reader.getAttrAsString("theme",StringUtils.EMPTY);
        item.tag = reader.getAttrAsString("tag",StringUtils.EMPTY);
        return item;
    }

    @Override
    public void writeAdditionalAttr(XMLPrintWriter writer) {
        writer.attr("name", "Theme").attr("class",DemoBackground.class.getName());
        if ( StringUtils.isNotEmpty(theme) ) {
            writer.attr("theme", theme);
        }
        if ( StringUtils.isNotEmpty(tag) ) {
            writer.attr("tag", tag);
        }
    }

    @Override
    /**
     * 根据指定的画图对象和尺寸来画背景，并且保存对应的单元格坐标点，背景预处理
     * @param g 画图对象
     * @param repo 模板上下文
     * @param cellPoint 单元格坐标
     * @param width 图片宽
     * @param height 图片高
     */
    public void preDealBackground(Graphics g, Repository repo, Point cellPoint, int width, int height){
        super.preDealBackground(g, repo, cellPoint, width, height);
        System.out.println();
    }

    @Override
    /**
     * 导出的时候更改背景
     * @param operate 导出的操作类型，excel,pdf,word...
     * @return 背景
     */
    public Background traverseForExport(ObjectHolder operate) {
        return this;
    }

    /**
     * 控件背景输出json到web端.
     *
     * @param repo 浏览器上下文
     * @return json格式内容.
     */
    @Override
    public JSONObject createJSONConfig(Repository repo) throws JSONException {
        Color color = getColor();
        if (color == null) {
            // 透明色, 不设置{background : ""}.
            return JSONObject.EMPTY;
        }
        String colorStr = StableUtils.javaColorToCSSColor(color);
        return JSONObject.create().put("background", colorStr);
    }

    @Override
    public void readXML(XMLableReader reader) {
        //这里实现得好奇怪
        super.readXML(reader);
    }

    @Override
    public void writeXML(XMLPrintWriter writer) {
        //这里实现得好奇怪
        super.writeXML(writer);
    }

    @Override
    public JSONObject createJSONConfig(Repository repo, int width, int height) throws JSONException{
        JSONObject jo = new JSONObject();
        jo.put("background-color", StableUtils.javaColorToCSSColor((this.getColor())));
        return jo;
    }

}
